﻿using Decisions.Core.UnitTests.Web;
using Decisions.Core.UnitTests.Web.PageModels;
using Decisions.Core.UnitTests.Web.PageModels.Controls;
using Decisions.Core.UnitTests.Web.PageModels.Report;
using NUnit.Framework;
using OpenQA.Selenium;
using OpenQA.Selenium.Chrome;
using OpenQA.Selenium.Remote;
using OpenQA.Selenium.Support.UI;
using System;
using System.Linq;

namespace TestsClass
{
    [TestFixture]
    public class TestClass
    {
        private RemoteWebDriver driver;
        private string baseURL;

        [SetUp]
        public void Init()
        {
            driver = new ChromeDriver();
            driver.Manage().Window.Maximize();
            baseURL = "http://localhost/decisions";
        }

        [Test]
        public void LoginTest()
        {
            //Navigate to login page
            driver.Navigate().GoToUrl($"{baseURL}/Login");

            IWebElement loginPage = driver.FindElement(By.ClassName("login"));

            //Check whether login page is diaplayed or not
            Assert.IsTrue(loginPage.Displayed, "Login Page is not displayed");

            //Enter username
            IWebElement userName = loginPage.FindElement(By.Id("txt_login_username"));
            userName.SendKeys("admin@decisions.com");

            //Enter password
            IWebElement password = loginPage.FindElement(By.Id("txt_login_password"));
            password.SendKeys("admin");

            //Click on login
            IWebElement loginButton = loginPage.FindElement(By.Id("btn_login_submit"));
            loginButton.Click();

            IWebElement portal_select_Page = driver.FindElement(By.ClassName("login_cnt"));

            //Click Studio
            IWebElement studio_loginButton = portal_select_Page.FindElement(By.Id("btn_designer_portal"));
            studio_loginButton.Click();

            //Checking whether portal gets open or not
            IWebElement portalHeader = driver.FindElement(By.Id("PageHeader"));

            //Wait for portal spinner
            WebDriverWait webDriverWait = new WebDriverWait(driver, new TimeSpan(0, 0, 5));
            webDriverWait.Until((driver) => { return portalHeader != null; });

            Assert.NotNull(portalHeader, "User does not gets loggedIn");
        }

        [Test]
        public void RunFlowTest()
        {
            string flowId = "e42d466a-4651-11ea-9ed8-902b34b80ee3";

            //you can use your credentials here
            string userId = "admin@decisions.com";
            string password = "admin";

            //run the flow          
            RunFlowPage runFlowPage = new RunFlowPage(driver, flowId, userId, password);
            runFlowPage.WaitForPortalSpinner(TimeSpan.FromSeconds(5));

            InteractWithFormComponents(runFlowPage);
        }

        [Test]
        public void RunFlowInSameWindowTest()
        {
            string flowName = "TestFlow";
            string folderId = "d3fc6f93-4651-11ea-9ed8-902b34b80ee3";

            //you can use your credentials here
            string userId = "admin@decisions.com";
            string password = "admin";

            //Navigate to the folder
            HomePage homePage = new HomePage(driver, folderId, "List", userId, password);

            //Waiting for portal spinner gone
            homePage.WaitForPortalSpinner(TimeSpan.FromSeconds(5));

            //Find ReportViewerPart
            ReportViewerPart reportViewerPart = homePage.GetComponentByID<ReportViewerPart>("aa22f024-79f0-4fad-b1bf-7d6771f6dba9");
            reportViewerPart.ReportViewer.ReportViewMode = ReportViewMode.Grid;

            //Find and select flow on report viewer
            reportViewerPart.ReportViewer.GridView.ContextClickOnEntity(flowName);

            //Get Action Layer
            IWebElement actionsLayer = driver.FindElement(By.ClassName("menu-holder-base"));

            //Asserting whether action layer is displayed or not
            Assert.IsTrue(actionsLayer.Displayed, "Actions is not displaying after right click on flow");

            //Find all actions after right click on flow
            IWebElement[] actions = actionsLayer.FindElements(By.TagName("li")).ToArray();

            Assert.IsNotNull(actions, "Actions is not displaying on action layer after right click on flow");

            IWebElement runflowAction = null;

            // WaitUntil method will wait until given condition satisfied
            homePage.WaitUntil(() =>
            {
                runflowAction = actions.FirstOrDefault(s => s.Text.Contains("Run Flow"));
                return runflowAction != null;
            });

            Assert.IsNotNull(runflowAction, " 'Run Flow' action is not displaying on action layer after right click on flow");

            //Click on Run Flow Action
            runflowAction.Click();

            InteractWithFormComponents(homePage);
        }

        private void InteractWithFormComponents(BasePage basePage)
        {
            //Finding textbox on form
            SilverTextBox textBox = basePage.GetComponentByID<SilverTextBox>("8576f123-09b7-45e9-8815-bdb043037e21");

            //Sending text in textbox
            textBox.Text = "admin";

            //Finding dropdownlist on form
            SilverCombo combobox = basePage.GetComponentByID<SilverCombo>("c141a842-089b-4b22-931a-d121f1162bad");

            //Selecting value in dropdownlist
            combobox.SelectItem("A");

            //Assert dropdown selected value
            Assert.AreEqual("A", combobox.SelectedItem(), $"Selected value in dropdown does not matched, Expected value: A and Actual value: {combobox.SelectedItem()}");

            //find and click on submit button
            SilverButton button = basePage.GetComponentByID<SilverButton>("077f7bf3-c458-4694-a4b8-94ac1028b653");
            button.Click();

            //Waiting for form submit
            basePage.WaitForPortalSpinner(TimeSpan.FromSeconds(5));

            // Assert mapped value on next form

            //Find label on next form
            SilverLabel textboxOnNextForm = basePage.GetComponentByID<SilverLabel>("7f80e0f9-c531-44b7-81af-6aa9f60a93bf");

            //Assert label value that coming from first form
            Assert.AreEqual("admin", textboxOnNextForm.Text, $"Textbox value on next form does not matched, Expected: 'admin' and Actual: {textboxOnNextForm.Text}");

            //Find second label on next form       
            SilverLabel selectedDropdownValueOnNextForm = basePage.GetComponentByID<SilverLabel>("883092f4-5615-483e-93a4-7121380f210b");

            //Assert label value that coming from first form
            Assert.AreEqual("A", selectedDropdownValueOnNextForm.Text, $"Selected dropdown value on next form does not matched, Expected: 'A' and Actual: {selectedDropdownValueOnNextForm.Text}");
        }


        [TearDown]
        public void Teardown()
        {
            driver.Quit();
        }
    }
}
